/*
* Copyright (C) 1998-2006 The Boeing Company
*                         Ed Trettevik <eat@nodebrain.org>
*
* NodeBrain is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place Suite 330, Boston, MA 02111-1307, USA.
*
*=============================================================================
* Program:  NodeBrain
*
* File:     nbapi.h
*
* Title:    NodeBrain C API Header
*
* Function:
*
*   This header provides the definitions needed for using NodeBrain's C API.
*   The API can be used to include the NodeBrain interpreter in your own
*   programs, extend the interpreter by adding expert skill modules, or
*   both.  See the NodeBrain User's Guide online at www.nodebrain.org for a
*   full description of the API.
*
* Synopsis:
*
*   #include "nbapi.h"  [For user programs and expert skill modules.]
*
*   #include "nb.h"     [For distributed programs and modules only.]
*
* Description:
*
*   The API is a set of functions NodeBrain provides for you to call, and a
*   set of methods (functions) that you write and let NodeBrain call.  We use
*   the term "method" simply to distinguish the functions the API calls from
*   the functions it provides.
*
*   We do not expose NodeBrain's internal data structures to the C compiler
*   when using this header.  After all, the idea of an API is to give you a
*   simplified and well defined interface to a complicated set of code.  We
*   have only two data types for use as parameters, arguments and return values.
*
*     nbCELL    - pointer to brain cell 
*     nbSET     - pointer to a member cell's position within a set of cells
*
*   Variables of these types should only be assigned through calls to API
*   functions or received as parameters to API methods.  You should never
*   assign values to these variables directly. (There may be times when the
*   NULL value could be assigned, but let's see if we can avoid that.)
*
* Bugs:         
*
*   For now, you are limited to one instance of NodeBrain within a process.
*   This will be cleaned up in the future to enable multiple instances of
*   NodeBrain concurrently within a process.  We have to move a bunch of 
*   global variables to a "stem cell" to avoid a collision at the global
*   variables.
*  
*=============================================================================
* Change History:
*
*    Date    Name/Change
* ---------- -----------------------------------------------------------------
* 2003/03/17 eat 0.5.2  Start
* 2003/12/09 eat 0.6.0  Included new API functions
* 2005/04/08 eat 0.6.2  Cleaned up and consolidated definitions from other headers
*=============================================================================
*/
#include "nbspine.h"    // spinal API

extern int  agent;            /* running as daemon */

#if defined(NB_HEADER)        /* Define API data types for internal use */ 
typedef NB_Cell *nbCELL;
#else                         /* Define simplified data types for API use */ 
#include "nbstd.h"            /* Include standard stuff for API use also */
typedef void*  nbCELL;        /* void the API cell pointer */
typedef void   NB_Object;     /* obsolete in API */
typedef void   NB_Cell;       /* obsolete in API */ 
typedef void   NB_Term;       /* obsolete in API */
typedef void   NB_Skill;      /* obsolete in API */  
typedef void** NB_Pointer;    /* obsolete in API */ 
#endif

typedef void*  nbSET;         /* Position pointer within a set of cells (e.g. lists) */

/* nbservice.c */

int nbService(int (*serviceMain)(int argc,char *argv[]),int argc,char *argv[]);

/* nbobject.c */

#define NB_TYPE_DISABLED     1     /* API Cell Types */
#define NB_TYPE_UNKNOWN      2
#define NB_TYPE_PLACEHOLDER  3
#define NB_TYPE_STRING       4
#define NB_TYPE_REAL         5
#define NB_TYPE_LIST         6 
#define NB_TYPE_TERM         7 
#define NB_TYPE_VERB         8

/* nbexpert.c */

#define NB_EXPERT_CONSTRUCT 1  /* Construct an expert object */
#define NB_EXPERT_ASSERT    2  /* Assert something to an expert */
#define NB_EXPERT_EVALUATE  3  /* Evaluate an expert object */
#define NB_EXPERT_COMPUTE   4  /* Compute an expert object */
#define NB_EXPERT_SOLVE     5  /* Solve an expert object */
#define NB_EXPERT_SHOW      6  /* Show an expert object */
#define NB_EXPERT_ENABLE    7  /* Eabled an expert object */
#define NB_EXPERT_DISABLE   8  /* Disable an expert */
#define NB_EXPERT_DESTROY   9  /* Destroy an expert */
#define NB_EXPERT_COMMAND  10  /* Interpret an expert object command */
#define NB_EXPERT_ALARM    11  /* Alarm an expert object */
#define NB_EXPERT_ALERT    12  /* Alert an expert object */

/* nbcell.c */

/* We need to replace these with references to NB_TYPE_* values */
 
#if defined(WIN32)
#if defined(_USRDLL)
__declspec(dllimport)
#else
__declspec(dllexport)
#endif
#endif
extern nbCELL NB_CELL_DISABLED;

#if defined(WIN32)
#if defined(_USRDLL)
__declspec(dllimport)
#else
__declspec(dllexport)
#endif
#endif
extern nbCELL NB_CELL_UNKNOWN;

#if defined(WIN32)
#if defined(_USRDLL)
__declspec(dllimport)
#else
__declspec(dllexport)
#endif
#endif
extern nbCELL NB_CELL_FALSE;

#if defined(WIN32)
#if defined(_USRDLL)
__declspec(dllimport)
#else
__declspec(dllexport)
#endif
#endif
extern nbCELL NB_CELL_TRUE;

/* nbstream.c */

#if defined(WIN32)
__declspec(dllexport)
#endif
extern void *nbStreamProducerOpen(nbCELL context,char *streamName,void *handle,       
  void (*handler)(nbCELL context,void *handle,char *topic,int state));

#if defined(WIN32)
__declspec(dllexport)
#endif
extern int nbStreamOpen(nbCELL context,char *streamName,void *session,void (*subscriber)(nbCELL context,void *session,char
*msg));

#if defined(WIN32)
__declspec(dllexport)
#endif
extern int nbStreamClose(nbCELL context,char *streamName,void *session,void (*subscriber)(nbCELL context,void *session,char
 *msg));

#if defined(WIN32)
__declspec(dllexport)
#endif
extern void nbStreamPublish(nbCELL producer,char *msg);

// nbassertion.c 

#if defined(WIN32)
_declspec (dllexport)
#endif               
extern int nbAssertionAddTermValue(nbCELL context,nbSET *set,nbCELL term,nbCELL cell);

#if defined(WIN32)
_declspec (dllexport)
#endif               
extern void nbAssert(nbCELL context,nbSET set);

#if defined(WIN32)
_declspec (dllexport)
#endif               
extern void nbAlert(nbCELL context,nbSET set);

// nbrule.c

#if defined(WIN32)
_declspec (dllexport)
#endif               
extern void nbActionAssert(nbCELL context,nbSET assertion);

#if defined(WIN32)
_declspec (dllexport)
#endif               
extern void nbActionCmd(nbCELL context,char *cmd,int option);

#if defined(WIN32)
_declspec (dllexport)
#endif               
extern void nbAction(nbCELL context,nbSET assertion,char *cmd,int option);

/* nblist.c */

#if defined(WIN32)
_declspec (dllexport)
#endif               
extern nbCELL nbListCreate(nbCELL context,char **cursor);

#if defined(WIN32)  
_declspec (dllexport)
#endif
extern void *nbListOpen(nbCELL context,nbCELL cell);

#if defined(WIN32)
_declspec (dllexport)
#endif
extern void *nbListInsertCell(nbCELL context,nbSET *setP,nbCELL cell);

#if defined(WIN32)                  
_declspec (dllexport)
#endif           
extern nbCELL nbListRemoveCell(nbCELL context,nbSET *setP);

#if defined(WIN32)      
_declspec (dllexport)
#endif
extern nbCELL nbListGetCell(nbCELL context,nbSET *setP);

#if defined(WIN32)                 
_declspec (dllexport)
#endif
extern nbCELL nbListGetCellValue(nbCELL context,nbSET *setP); 

#if defined(WIN32)                 
_declspec (dllexport)
#endif
extern void nbListEmpty(nbCELL context,nbSET *set);

/* nbout.c */

#if defined(WIN32)
__declspec(dllexport)
#endif
extern int nbMsg(nbCELL context,int msgNumber,char msgType,char *format,...);

#if defined(WIN32)
__declspec(dllexport)
#endif
extern int nbPut(nbCELL context,char *format,...);                  

#if defined(WIN32)
__declspec(dllexport)
#endif
extern int nbDump(nbCELL context,void *data,int len);

#if defined(WIN32)
__declspec(dllexport)
#endif
extern int nbOutputHandlerAdd(nbCELL context,void *session,void (*handler)(nbCELL context,void *session,char *buffer));

#if defined(WIN32)
__declspec(dllexport)
#endif
extern int nbOutputHandlerRemove(nbCELL context,void *session,void (*handler)(nbCELL context,void *session,char *buffer));

#if defined(WIN32)
__declspec(dllexport)
#endif
extern void nbFlush();

/* nbsynapse.c */

#if defined(WIN32)
__declspec(dllexport)
#endif                  
extern nbCELL nbSynapseOpen(nbCELL cell,void *handle,void (*handler)(nbCELL cell,void *handle));

#if defined(WIN32)
__declspec(dllexport)
#endif
extern void *nbSynapseClose(nbCELL synapse);

/* nbtranslate.c */

#if defined(WIN32)
_declspec (dllexport)
#endif
extern nbCELL nbLoadTranslator(nbCELL context,char *file);                

#if defined(WIN32)
_declspec (dllexport)   
#endif
extern void nbTranslate(nbCELL context,char *source,nbCELL translator,nbCELL symContext,FILE *pipe);

/* nbterm.c */

#if defined(WIN32)  
_declspec (dllexport)
#endif
extern nbCELL nbTermCreate(nbCELL context,char *identifier,nbCELL definition);
#if defined(WIN32)
_declspec (dllexport)
#endif
extern nbCELL nbTermLocate(nbCELL context,char *identifier);
#if defined(WIN32)                  
_declspec (dllexport)
#endif           
extern nbCELL nbTermGetDefinition(nbCELL context,nbCELL term);
#if defined(WIN32)      
_declspec (dllexport)
#endif
extern char *nbTermGetName(nbCELL context,nbCELL term);
#if defined(WIN32)
_declspec (dllexport)
#endif
extern nbCELL nbTermSetDefinition(nbCELL context,nbCELL term,nbCELL definition);

/* nbcmd.c */

/* Command Control Options
*     
*    ECHO  - use if you want the command to be displayed to the log
*    HUSH  - use if command was displayed before calling nbCmd()
*            HUSH trumps ECHO 
*    TRACE - use if you want debugging info displayed
*            TRACE trumps HUSH
*/
#define NB_CMDOPT_ECHO      1 // Request echo
#define NB_CMDOPT_HUSH      2 // Suppress echo - already echoed
#define NB_CMDOPT_TRACE     4 // Display debugging info
#define NB_CMDOPT_NOSYM     8 // Suppress symbolic substitution
#define NB_CMDOPT_ALERT   128 // Indicates an action assertion is an alert

#if defined(WIN32)
__declspec(dllexport)
#endif
extern void nbCmd(nbCELL context,char *cursor,int option);


/* nbapi.c */

#if defined(WIN32)
__declspec(dllexport) 
#endif
extern nbCELL nbStart(int argc,char *argv[]);

#if defined(WIN32)
__declspec(dllexport) 
#endif
extern int nbServe(nbCELL context,int argc,char *argv[]);

#if defined(WIN32)
__declspec(dllexport) 
#endif
extern int nbStop(nbCELL context);

/* 0.6.0 API starts here */

#if defined(WIN32)
_declspec (dllexport)
#endif
extern int nbSkillDeclare(nbCELL context,void *(*bindFunction)(),void *moduleHandle,char *moduleName,char *skillName,nbCELL arglist,char *text);

#if defined(WIN32)
_declspec (dllexport)
#endif
extern int nbSkillSetMethod(nbCELL context,nbCELL skill,int methodId,void *method);


#define NB_SHOW_ITEM   0 /* show expert as single line item */
#define NB_SHOW_REPORT 1 /* show expert as multi-line report */

// Cell API Functions

#if defined(WIN32)
__declspec(dllexport)
#endif
extern nbCELL nbCellCreate(nbCELL context,char *cellExpression);

#if defined(WIN32)
__declspec(dllexport)
#endif
extern nbCELL nbCellParse(nbCELL context,char **cellExpression);

#if defined(WIN32)
__declspec(dllexport)
#endif
extern nbCELL nbCellGetValue(nbCELL context,nbCELL cell);

#if defined(WIN32)
__declspec(dllexport)
#endif                 
extern void nbCellEnable(nbCELL pub,NB_Object *sub);

#if defined(WIN32)                  
__declspec(dllexport)
#endif           
extern void nbCellDisable(nbCELL pub,NB_Object *sub);

#if defined(WIN32)
__declspec(dllexport)
#endif
extern int nbCellGetType(nbCELL context,nbCELL cell);
#if defined(WIN32)
__declspec(dllexport) 
#endif
extern nbCELL nbCellGrab(nbCELL context,nbCELL cell);
#if defined(WIN32)
__declspec(dllexport) 
#endif
extern nbCELL nbCellDrop(nbCELL context,nbCELL cell);
#if defined(WIN32)
__declspec(dllexport) 
#endif
extern void nbCellRelease(nbCELL context,nbCELL cell);
#if defined(WIN32)
__declspec(dllexport) 
#endif
extern void nbCellPub(nbCELL context,nbCELL cell);
#if defined(WIN32)
__declspec(dllexport) 
#endif
extern void nbCellFree(nbCELL context,nbCELL cell);
#if defined(WIN32)
__declspec(dllexport) 
#endif
extern void nbCellShow(nbCELL context,nbCELL cell);
#if defined(WIN32)
__declspec(dllexport) 
#endif
extern nbCELL nbCellEvaluate(nbCELL context,nbCELL cell);
#if defined(WIN32)
__declspec(dllexport) 
#endif
extern nbCELL nbCellCompute(nbCELL context,nbCELL cell);
#if defined(WIN32)
__declspec(dllexport) 
#endif
extern nbCELL nbCellSolve(nbCELL context,nbCELL cell);
/* API String Functions */
#if defined(WIN32)
__declspec(dllexport)
#endif
extern nbCELL nbStringCreate(nbCELL context,char *string);
#if defined(WIN32)
__declspec(dllexport) 
#endif
extern char *nbStringGet(nbCELL context,nbCELL cell);

/* API Real Functions */
#if defined(WIN32)
__declspec(dllexport) 
#endif
extern nbCELL nbRealCreate(nbCELL context,double real);
#if defined(WIN32)
__declspec(dllexport) 
#endif
extern double nbRealGet(nbCELL context,nbCELL cell);

#if defined(WIN32)
__declspec(dllexport) 
#endif
extern int nbExpertSetLevel(nbCELL context,nbCELL cell);

#if defined(WIN32)
__declspec(dllexport) 
#endif
extern void nbExpertSetValue(nbCELL context,nbCELL cell);

/* 0.6.2 additions to API */
/* Listeners */

#if defined(WIN32)
_declspec (dllexport)
#endif
extern int nbListenerEnableOnDaemon(nbCELL context);

#if defined(WIN32)
_declspec (dllexport)
#endif
extern int nbListenerAdd(nbCELL context,int fildes,void *session,void (*handler)(nbCELL context,int fildes,void *));

#if defined(WIN32)
_declspec (dllexport)
#endif
extern int nbListenerReplace(nbCELL context,int fildes,void *session,void (*handler)(nbCELL context,int fildes,void *));

#if defined(WIN32)
_declspec (dllexport)
#endif
extern int nbListenerRemove(nbCELL context,int fildes);

#if defined(WIN32)
_declspec (dllexport)
#endif
extern int nbListenerStart(nbCELL context);

#if defined(WIN32)
_declspec (dllexport)
#endif
extern int nbListenerStop(nbCELL context);

/* nbip.c */

#if defined(WIN32)
_declspec (dllexport)
#endif
extern char *nbIpGetName(unsigned int ipaddr,char *name,int len);

#if defined(WIN32)
_declspec (dllexport)
#endif
extern unsigned int nbIpGetUdpServerSocket(nbCELL context,char *interfaceAddr,unsigned short port);

#if defined(WIN32)
_declspec (dllexport)                  
#endif
extern int nbIpGetDatagram(nbCELL context,int socket,unsigned int *raddr,unsigned short *rport,unsigned char *buffer,size_t length);

#if defined(WIN32)
_declspec (dllexport)
#endif
extern char *nbIpGetAddrString(char *addr,unsigned int address);

#if defined(WIN32)
_declspec (dllexport)
#endif
extern int nbIpGetSocketAddrString(int socket,char *ipaddr);

// nbmedulla.c

#if !defined(NB_HEADER)
#include "nbspine.h"       // this is weird

typedef void*  nbPROCESS;  // void the API process pointer 

#if defined(WIN32)
_declspec (dllexport)
#endif
extern nbPROCESS nbMedullaProcessOpen(int options,char *cmd,char *log,void *session,
  int (*producer)(nbPROCESS process,int pid,void *session),
  int (*consumer)(nbPROCESS process,int pid,void *session,char *msg),
  int (*logger)(nbPROCESS process,int pid,void *session,char *msg),
  char *msgbuf);
#if defined(WIN32)
_declspec (dllexport)
#endif
extern int nbMedullaProcessPid(nbPROCESS process);
#if defined(WIN32)
_declspec (dllexport)
#endif
extern nbPROCESS nbMedullaProcessAdd(int pid,char *cmd);
#if defined(WIN32)
_declspec (dllexport)
#endif
extern nbPROCESS nbMedullaProcessFind(int pid);
#if defined(WIN32)
_declspec (dllexport)
#endif
extern int nbMedullaProcessPut(nbPROCESS process,char *buffer);
#if defined(WIN32)
_declspec (dllexport)
#endif
extern int nbMedullaProcessTerm(nbPROCESS process);
#if defined(WIN32)
_declspec (dllexport)
#endif
extern nbPROCESS nbMedullaProcessClose(nbPROCESS process);

#endif
